
/**********************************************************************************************
    This file is part of PriiDash(TM), an enhanced instrument panel and
    data logger for the Prius and more.
    Copyright (C) 2011 by TeamPrii at sourceforge.net

    PriiDash(TM) is provided "AS IS" under the terms of
    the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version, WITH ADDITIONAL TERMS in
    paragraphs below.

    This program is distributed in the hope that it will be useful,
    but Without Any Warranty; without even the implied warranty of
    Merchantability or Fitness For A Particular Purpose.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    ADDITIONAL TERMS

    IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, DEATH; INJURY; COLLISION AND/OR DAMAGE TO VEHICLES;
    LIABILITY FOR ACCIDENTS AND/OR TRAFFIC VIOLATIONS; PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE OR
    INABILITY TO USE THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHOR RESERVES ALL RIGHTS OF OWNERSHIP OF THE NAME OF THE SOFTWARE, PriiDash(TM).
    ALL MODIFIED VERSIONS MUST BE CLEARLY MARKED AS BEING DIFFERENT FROM THE ORIGINAL,
    AND MUST CARRY ACKNOWLEDGEMENT TO THE ORIGINAL AUTHOR, TeamPrii at sourceforge.net.
    ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE
    MUST DISPLAY THE FOLLOWING ACKNOWLEDGEMENT:
    This product includes the software PriiDash(TM) developed by
    TeamPrii at sourceforge.net.

    To contact the author: look up TeamPrii at sourceforge.net.
    Project home page: http://priidash.sourceforge.net/
************************************************************************************************/

#ifndef _UI_H
#define	_UI_H
#include <string>
#include "OS.h"
#ifdef VC
extern int g_baudRate;
#else
#include <termios.h> /* POSIX terminal control definitions */
extern speed_t g_baudRate;
#endif
const char xmlfilename[] = "INI_Settings.xml";
const double literToUSGal = 0.264172052;
const double literToUKGal = 0.219969157;
extern double g_literToGal; // for MPG calculation
extern double g_literToGalCost; // unit factor for fuel cost calculation
extern double g_km2mile;
enum UIstateType {Stopped, Running}; // define a new type for UI state
extern UIstateType UIstate; // Defined in UI.cpp, Controlled by the Go button in Dash.fl -> Dash.cxx & Dash.h
enum UIrunType {Normal, Simulation}; // define a new type for UI running type
extern UIrunType UIruntype;
enum GasVolUnit {USGal, UKGal, Liter};
struct INI_Settings{
    std::string portname; // OBDUSB portname
    GasVolUnit gasvolunit, gascostunit; // fuel volume unit for MPG and cost, respectively
    int baudchoice; // baud rate choice, 0 = 2M, 1 = 1M, 2 = 500K, 3 = 115200
    int odounit; // distance unit, 0 = mile, 1 = km
    double fuelcost; // fuel cost per unit volume
    double odo; // ODO reading
    void load(const std::string &filename);
    void save(const std::string &filename);
    void reset(const std::string &filename); // reset to default values
};
extern INI_Settings ini; //declared in UI.cpp
extern char LicText[];
void init_filenames(); // initialize log and csv filenames, load settings from xml file
void setLiterToGal(GasVolUnit gvu, double& g_x); // set g_literToGal or g_literToGalCost according to ini.gasvolunit or ini.gascostunit
void setkm2mile(); // set g_km2mile according to ini.odounit
void setbaudRate(); // set g_baudRate according to ini.baudchoice
void setUIstate(UIstateType UIst);
void runUI();
void showLic();
#endif	/* _UI_H */

